from PyQt4 import QtCore, QtGui
from Ui_SpeechRecognitionWidget import Ui_SpeechRecognitionWidget
from SpeechRecognition import SpeechRecognition
from SpeechRecognition import KeyWordDetection
from SpeechRecognition import SR_Word
from Ui_KeyWordDetection import Ui_KeyWordDetection
import sys
    
class SpeechRecognitionWidget(QtGui.QWidget,Ui_SpeechRecognitionWidget):
    
    def __init__(self,parent):
        QtGui.QWidget.__init__(self,parent)
        self.setupUi(self)
        self.SR = None
        
    def setSpeechRecognizer(self,speechRecognizer):
        self.SR = speechRecognizer
        self.SR.addCharacterEvent(self.characterHandler)
        
        self.SR.addWordEvent(self.wordHandler)
        
        self.SR.addSentenceEvent(self.sentenceHandler)    
     
    def getInputField(self):
        return self.InputLineEdit 
        
    def characterHandler(self,charObject):
        self.CharacterLineEdit.setText(charObject.getCharacter())
        
    def wordHandler(self,word):
        self.WordLineEdit.setText(word.getWord())
        
    def sentenceHandler(self,sentence):
        self.SentenceList.insertItem(0,sentence.getSentence())
        
        
    def closeEvent(self):
        self.SR.quit()

class KeyWordDetectionWidget(QtGui.QWidget,Ui_KeyWordDetection):
    
    def __init__(self,parent):
        QtGui.QWidget.__init__(self,parent)
        self.setupUi(self)
        self.KD = None
        self.SrWordList = []
        self.detected = []
        
        #add functions
        self.connect(self.AddWordButton,QtCore.SIGNAL('clicked()'),self.addWord)
        self.connect(self.AddAlternativeButton,QtCore.SIGNAL('clicked()'),self.addAlternative)
        self.connect(self.ClearAllButton,QtCore.SIGNAL('clicked()'),self.clearAll)
        self.connect(self.RemoveButton,QtCore.SIGNAL('clicked()'),self.removeWord)
        self.connect(self.StartButton,QtCore.SIGNAL('clicked()'),self.startRun)
        self.connect(self.StopButton,QtCore.SIGNAL('clicked()'),self.stopRun)
                
        #set enables
        self.clearList()
        self.AddWordButton.setEnabled(False)
        self.WordLineEdit.setEnabled(False)
        self.PercentagePlaceholder.setText("0.0%")
    
    def clearList(self):    
        self.AddAlternativeButton.setEnabled(False)
        self.AlternativeLineEdit.setEnabled(False)
                
        self.RemoveButton.setEnabled(False)
        self.ClearAllButton.setEnabled(False)
        
        self.StartButton.setEnabled(False)
        self.StopButton.setEnabled(False)
        
        self.WordList.clear()
        self.SrWordList = []
          
    def addWord(self):
        word = str(self.WordLineEdit.text())
        if not word == None and len(word) > 0:
            #set enables
            self.AddAlternativeButton.setEnabled(True)
            self.AlternativeLineEdit.setEnabled(True)
            
            self.RemoveButton.setEnabled(True)
            self.ClearAllButton.setEnabled(True)
            
            self.StartButton.setEnabled(True)
            
            srWord = SR_Word(word,[],len(self.SrWordList))
            self.SrWordList.append(srWord)
            
            self.WordLineEdit.setText("")
            
            self.refreshList()
            
        self.WordLineEdit.setFocus()
            
    def refreshList(self):
        self.WordList.clear()
        
        for srWord in self.SrWordList:
            word = str(srWord)
            self.WordList.addItem(word)
    
    def addAlternative(self):
        alternative = str(self.AlternativeLineEdit.text())
        if not alternative == None and len(alternative) > 0:
            indexes = self.WordList.selectedIndexes()
            if len(indexes) > 0:
                for index in indexes:
                    row = index.row()
                    srWord = self.SrWordList[row]
                    srWord.addAlternative(alternative)
                
                self.AlternativeLineEdit.setText("")
                self.refreshList()
    
    def clearAll(self):
        self.clearList()
        
    def removeWord(self):
        indexes = self.WordList.selectedIndexes()
        if len(indexes) > 0:
            del self.SrWordList[indexes[0].row()]
            
            if len(self.SrWordList) == 0:
                self.clearList()
            else:
                self.refreshList()
    
    def startRun(self):
        self.AddWordButton.setEnabled(False)
        self.WordLineEdit.setEnabled(False)
        
        self.AddAlternativeButton.setEnabled(False)
        self.AlternativeLineEdit.setEnabled(False)
                
        self.RemoveButton.setEnabled(False)
        self.ClearAllButton.setEnabled(False)
        
        self.StartButton.setEnabled(False)
        self.StopButton.setEnabled(True)
        
        self.WordList.clear()
        
        self.detected = []
        
        self.KD.setKeyWordRecognition(self.wordFound,self.SrWordList)
        self.KD.continueRun()
        
    def stopRun(self):
        
        self.KD.pauseRun()
        self.refreshList()
        
        self.AddWordButton.setEnabled(True)
        self.WordLineEdit.setEnabled(True)
        
        self.AddAlternativeButton.setEnabled(True)
        self.AlternativeLineEdit.setEnabled(True)
                
        self.RemoveButton.setEnabled(True)
        self.ClearAllButton.setEnabled(True)
        
        self.StartButton.setEnabled(True)
        self.StopButton.setEnabled(False)
        
        self.PercentagePlaceholder.setText("0.0%")
        
    
    def wordFound(self,Internal_Word,stringWord,foundStringWord,SR_Word):
        word = stringWord+" ("+foundStringWord+")"
        self.WordList.insertItem(0,word)
        
        for srWord in self.SrWordList:
            if srWord.getWord() == SR_Word.getWord():
                try:
                    self.detected.index(srWord)
                    
                except:
                    self.detected.append(srWord)
        per = float(len(self.detected))/float(len(self.SrWordList))
        self.PercentagePlaceholder.setText(str(int(per*100))+"%")
            
        
    def setKeyWordDetecter(self,keyWordDetector):
        self.KD = keyWordDetector
        self.KD.pauseRun()
        
        self.AddWordButton.setEnabled(True)
        self.WordLineEdit.setEnabled(True)
        
if __name__ == '__main__':
    app = QtGui.QApplication(sys.argv)
    main = QtGui.QMainWindow()
    main.show()
    
    mainWidget = QtGui.QWidget(main)
    verticalLayout = QtGui.QHBoxLayout(mainWidget)
    SRWidget = SpeechRecognitionWidget(mainWidget)
    verticalLayout.addWidget(SRWidget)
    SR = SpeechRecognition(False,SRWidget.getInputField())
    SRWidget.setSpeechRecognizer(SR)
    
    KDWidget = KeyWordDetectionWidget(mainWidget)
    verticalLayout.addWidget(KDWidget)
    KD = KeyWordDetection(SR)
    KDWidget.setKeyWordDetecter(KD)
    
    main.setCentralWidget(mainWidget)
    
    
    
    sys.exit(app.exec_())    
        
        
        